<?php

namespace App\Controller\Sub;

use App\Http\HttpRequest;
use App\Http\HttpResponse;
use App\Http\ViewHttpResponse;
use App\Model\AuthModel;
use App\Session\MutableSessionHandle;
use App\Validation\FieldValidationFail;
use App\Validation\ValidationFail;
use App\Validation\Validators;

class AuthController {
    private AuthModel $model;

    /**
     * @param AuthModel $model
     */
    public function __construct(AuthModel $model) {
        $this->model = $model;
    }

    public function displayRegister(): HttpResponse {
        return ViewHttpResponse::twig("display_register.html.twig", []);
    }

    /**
     * @param string $viewName
     * @param ValidationFail[] $fails
     * @return HttpResponse
     */
    private function displayBadFields(string $viewName, array $fails): HttpResponse {
        $bad_fields = [];
        foreach ($fails as $err) {
            if ($err instanceof FieldValidationFail) {
                $bad_fields[] = $err->getFieldName();
            }
        }
        return ViewHttpResponse::twig($viewName, ['bad_fields' => $bad_fields]);
    }

    /**
     * @param mixed[] $request
     * @param MutableSessionHandle $session
     * @return HttpResponse
     */
    public function confirmRegister(array $request, MutableSessionHandle $session): HttpResponse {
        $fails = [];
        $request = HttpRequest::from($request, $fails, [
            "username" => [Validators::name(), Validators::lenBetween(2, 32)],
            "password" => [Validators::lenBetween(6, 256)],
            "confirmpassword" => [Validators::lenBetween(6, 256)],
            "email" => [Validators::regex("/^\\S+@\\S+\\.\\S+$/"), Validators::lenBetween(5, 256)],
        ]);
        if (!empty($fails)) {
            return $this->displayBadFields("display_register.html.twig", $fails);
        }
        $account = $this->model->register($request['username'], $request["password"], $request['confirmpassword'], $request['email'], $fails);
        if (!empty($fails)) {
            return $this->displayBadFields("display_register.html.twig", $fails);
        }

        $session->setAccount($account);

        $target_url = $session->getInitialTarget();
        return HttpResponse::redirect($target_url ?? "/home");
    }


    public function displayLogin(): HttpResponse {
        return ViewHttpResponse::twig("display_login.html.twig", []);
    }

    /**
     * @param mixed[] $request
     * @return HttpResponse
     */
    public function confirmLogin(array $request, MutableSessionHandle $session): HttpResponse {
        $fails = [];
        $request = HttpRequest::from($request, $fails, [
            "password" => [Validators::lenBetween(6, 256)],
            "email" => [Validators::regex("/^\\S+@\\S+\\.\\S+$/"), Validators::lenBetween(5, 256)],
        ]);
        if (!empty($fails)) {
            return $this->displayBadFields("display_login.html.twig", $fails);
        }

        $account = $this->model->login($request['email'], $request['password'], $fails);
        if (!empty($fails)) {
            return $this->displayBadFields("display_login.html.twig", $fails);
        }

        $session->setAccount($account);

        $target_url = $session->getInitialTarget();
        $session->setInitialTarget(null);
        return HttpResponse::redirect($target_url ?? "/home");
    }

}
