<?php

namespace App\Model;

use App\Data\Account;
use App\Gateway\AccountGateway;
use App\Validation\FieldValidationFail;
use App\Validation\ValidationFail;

class AuthModel {
    private AccountGateway $gateway;

    /**
     * @param AccountGateway $gateway
     */
    public function __construct(AccountGateway $gateway) {
        $this->gateway = $gateway;
    }


    /**
     * @param string $username
     * @param string $password
     * @param string $confirmPassword
     * @param string $email
     * @param ValidationFail[] $failures
     * @return Account|null the registered account or null if failures occurred
     */
    public function register(string $username, string $password, string $confirmPassword, string $email, array &$failures): ?Account {

        if ($password != $confirmPassword) {
            $failures[] = new FieldValidationFail("confirmpassword", "password and password confirmation are not equals");
        }

        if ($this->gateway->exists($email)) {
            $failures[] = new FieldValidationFail("email", "email already exist");
        }

        if (!empty($failures)) {
            return null;
        }

        $hash = password_hash($password, PASSWORD_DEFAULT);

        $token = $this->generateToken();
        $accountId = $this->gateway->insertAccount($username, $email, $token, $hash);
        return new Account($email, $username, $token, $accountId);
    }

    private function generateToken(): string {
        return base64_encode(random_bytes(64));
    }

    /**
     * @param string $email
     * @param string $password
     * @param ValidationFail[] $failures
     * @return Account|null the authenticated account or null if failures occurred
     */
    public function login(string $email, string $password, array &$failures): ?Account {
        if (!$this->gateway->exists($email)) {
            $failures[] = new FieldValidationFail("email", "email doesnt exists");
            return null;
        }
        $hash = $this->gateway->getHash($email);

        if (!password_verify($password, $hash)) {
            $failures[] = new FieldValidationFail("password", "invalid password");
            return null;
        }

        return $this->gateway->getAccountFromMail($email);
    }


}
